<?php
/**
 * Ce fichier contient la configuration et l'ensemble des fonctions implémentant le service AccuWeather (accuweather).
 * Ce service est capable de fournir des données au format JSON.
 *
 * @package SPIP\RAINETTE\SERVICES\ACCUWEATHER
 */
if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}


// ------------------------------------------------------------------------------------------------
// Les fonctions qui suivent définissent l'API standard du service et sont appelées par la fonction
// unique de chargement des données météorologiques `meteo_charger()`.
// ------------------------------------------------------------------------------------------------

/**
 * Construit l'url de la requête correspondant au lieu, au type de données et à la configuration utilisateur
 * du service (par exemple, le code d'inscription, le format des résultats...).
 *
 * @api
 * @uses langue_determiner()
 * @uses lieu_normaliser()
 *
 * @param string $lieu
 *        Lieu pour lequel on acquiert les données météorologiques.
 * @param string $mode
 *        Type de données météorologiques. Les valeurs possibles sont `infos`, `conditions` ou `previsions`.
 * @param int    $periodicite
 *        La périodicité horaire des prévisions :
 *        - `24`, `12`, `6`, `3` ou `1`, pour le mode `previsions`
 *        - `0`, pour les modes `conditions` et `infos`
 * @param array  $configuration
 *        Configuration complète du service, statique et utilisateur.
 *
 * @return string
 *        URL complète de la requête.
 */
function accuweather_service2url($lieu, $mode, $periodicite, $configuration) {

	// Identification de la langue du resume.
	include_spip('inc/rainette_normaliser');
	$code_langue = langue_determiner($configuration);

	// On normalise le lieu et on récupère son format.
	// Le service accepte uniquement son propre keyLocation id sous forme d'un entier.
	$lieu_normalise = lieu_normaliser($lieu);

	// On construit le mode
	if ($mode === 'infos') {
		$demande = 'locations';
	} elseif ($mode === 'conditions') {
		$demande = 'currentconditions';
	} else {
		$demande = 'forecasts';
	}

	return
		$configuration['endpoint_requete']
		. $demande
		. '/v1/'
		. ($mode === 'previsions' ? 'daily/5day/' : '')
		. $lieu_normalise
		. ".{$configuration['format_flux']}"
		. '?apikey=' . $configuration['inscription']
		. '&language=' . $code_langue
		. ($configuration['unite'] === 'm' ? '&metric=true' : '')
		. '&details=true';
}


/**
 * @param array $erreur
 *
 * @return bool
 */
function accuweather_erreur_verifier($erreur) {

	// Initialisation
	$est_erreur = false;

	// Une erreur est uniquement décrite par un message.
	if (!empty($erreur['code']) or !empty($erreur['message'])) {
		$est_erreur = true;
	}

	return $est_erreur;
}


/**
 * Complète par des données spécifiques au service le tableau des conditions issu
 * uniquement de la lecture du flux.
 *
 * @api
 *
 * @param array $tableau
 *        Tableau standardisé des conditions contenant uniquement les données fournies sans traitement
 *        par le service.
 * @param array $configuration
 *        Configuration complète du service, statique et utilisateur.
 *
 * @return array
 *        Tableau standardisé des conditions météorologiques complété par les données spécifiques
 *        du service.
 */
function accuweather_complement2conditions($tableau, $configuration) {

	static $tendances = array('S' => 'steady', 'R' => 'rising', 'F' => 'falling', '' => '');

	if ($tableau) {
		// Correspondance des tendances de pression dans le système standard
		$tableau['tendance_pression'] = $tendances[$tableau['tendance_pression']];

		// Compléter le tableau standard avec les états météorologiques calculés
		etat2resume_accuweather($tableau, $configuration);
	}

	return $tableau;
}


/**
 * Modifie le flux jour des prévisions issu directement du service pour qu'il puisse être conforme
 * à la logique de normalisation qui s'en suivra.
 *
 * Pour AccuWeather, pour la périodicité 12h on modifie les index 'Day' et 'Night' qui contiennent les flux
 * horaires par un tableau de base 'hourly' dont les index sont numériques : 0 jour et 1 nuit.
 *
 * @api
 *
 * @param array $flux          Flux jour non normalisé provenant du service
 * @param int   $periodicite   Périodicité horaire des prévisiosns (24, 12, 6, 3 ou 1)
 * @param array $configuration Configuration complète du service, statique et utilisateur.
 *
 * @return array Flux jour modifié
 */
function accuweather_flux2previsions($flux, $periodicite, $configuration) {

	if (
		$flux
		and $periodicite === 12
	) {
		// On extrait les index 'Day' et 'Night' que l'on affecte à un index 'hourly' et on les supprime.
		$flux['Hourly'] = array(
			$flux['Day'],
			$flux['Night']
		);
		unset($flux['Day'], $flux['Night']);
	}

	return $flux;
}


/**
 * Complète par des données spécifiques au service le tableau des conditions issu
 * uniquement de la lecture du flux.
 *
 * @api
 *
 * @param array $tableau
 *        Tableau standardisé des conditions contenant uniquement les données fournies sans traitement
 *        par le service.
 * @param array $configuration
 *        Configuration complète du service, statique et utilisateur.
 * @param int   $index_periode
 *        Index où trouver et ranger les données. Cet index n'est pas utilisé pour les conditions
 *
 * @return array
 *        Tableau standardisé des conditions météorologiques complété par les données spécifiques
 *        du service.
 */
function accuweather_complement2previsions($tableau, $configuration, $index_periode) {

	if (($tableau) and ($index_periode > -1)) {
		// Compléter le tableau standard avec les états météorologiques calculés
		etat2resume_accuweather($tableau, $configuration);
	}

	return $tableau;
}


/**
 * ---------------------------------------------------------------------------------------------
 * Les fonctions qui suivent sont des utilitaires uniquement appelées par les fonctions de l'API
 * ---------------------------------------------------------------------------------------------
 */

/**
 * Calcule les états en fonction des états météorologiques natifs fournis par le service.
 *
 * @internal
 *
 * @param array $tableau
 *        Tableau standardisé des conditions contenant uniquement les données fournies sans traitement
 *        par le service. Le tableau est mis à jour et renvoyé à l'appelant.
 * @param array $configuration
 *        Configuration complète du service, statique et utilisateur.
 *
 * @return void
 */
function etat2resume_accuweather(&$tableau, $configuration) {

	if ($tableau['code_meteo']) {
		// Determination de l'indicateur jour/nuit qui permet de choisir le bon icone
		// Pour ce service aucun indicateur n'est disponible
		// -> on utilise l'identfiant jour:nuit fourni par le service
		if ($tableau['jour_meteo']) {
			// C'est le jour
			$tableau['periode'] = 0;
		} else {
			// C'est la nuit
			$tableau['periode'] = 1;
		}

		// Détermination du résumé à afficher.
		// Depuis la 3.4.6 on affiche plus que le résumé natif de chaque service car les autres services
		// que weather.com possèdent de nombreuses traductions qu'il convient d'utiliser.
		// Pour éviter de modifier la structure de données, on conserve donc desc_meteo et resume même si
		// maintenant ces deux données coincident toujours.
		$tableau['resume'] = ucfirst($tableau['desc_meteo']);

		// Determination de l'icone qui sera affiché.
		// -- on stocke le code afin de le fournir en alt dans la balise img
		$tableau['icone'] = array();
		$tableau['icone']['code'] = $tableau['code_meteo'];
		// -- on calcule le chemin complet de l'icone.
		if ($configuration['condition'] == $configuration['alias']) {
			// On affiche l'icône natif fourni par le service et désigné par son url
			// en faisant une copie locale dans IMG/.
			include_spip('inc/distant');
			$tableau['icone']['source'] = copie_locale($tableau['icon_meteo']);
		} else {
			include_spip('inc/rainette_normaliser');
			if ($configuration['condition'] == "{$configuration['alias']}_local") {
				// On affiche un icône d'un thème local compatible avec AccuWeather.
				$chemin = icone_local_normaliser(
					"{$tableau['code_meteo']}.png",
					$configuration['alias'],
					$configuration['theme_local']
				);
			} else {
				// On affiche l'icône correspondant au code météo transcodé dans le système weather.com.
				$chemin = icone_weather_normaliser(
					$tableau['code_meteo'],
					$configuration['theme_weather'],
					$configuration['transcodage_weather'],
					$tableau['periode']
				);
			}
			include_spip('inc/utils');
			$tableau['icone']['source'] = find_in_path($chemin);
		}
	}
}
